-- Nature (Minetest 0.4 mod)
-- Nature is slowly capturing the world!
-- Licenced under the GPLv3

local ABM_DELAY = 7200

local TREE_GROW_DELAY = ABM_DELAY -- 2 hrs
local TREE_GROW_CHANCE = 15 -- * 0.5%
local DENSITY = 3 -- allow 3 trunks in one 'cross'
local MINIMUM_LIGHT = 8 -- light needed for the trees to grow

local ABM_CHANCE = 0.4

-- NODES

-- Irontree node
minetest.register_node("nature:irontree", {
    description = "Irontree",
    tile_images = {"nature_irontree_top.png", "nature_irontree_top.png", "nature_irontree.png"},
    is_ground_content = true,
    material = minetest.digprop_woodlike(1.0),
})

-- CRAFTS

-- Irontree
minetest.register_craft({
    output = 'nature:irontree 1',
    recipe = {
        { 'default:iron_lump 1' },
        { 'default:tree 1' },
    }
})

minetest.register_craft({
    type = "fuel",
    recipe = "nature:irontree",
    burntime = 30,
})

minetest.register_craft({
    type = "cooking",
    output = "default:iron_lump",
    recipe = "nature:irontree",
    cooktime = 4,
})

-- ABMs

-- Tree growing
minetest.register_abm({
    nodenames = { "default:leaves" },
    interval = TREE_GROW_DELAY,
    chance = ABM_CHANCE,

    action = function(pos, node, active_object_count, active_object_count_wider)
        -- Individual chance for different nodes
        if(math.random(1, 1000) > TREE_GROW_CHANCE) then
            return
        end

        if(minetest.env:get_node_light(pos, nil) < MINIMUM_LIGHT) then
            return
        end
        
        local trunk_count = 0
        local jungle_trunk_count = 0
        local iron_trunk_count = 0
        
        -- Check for trunks in area
        for i = -1, 1 do
        for j = -1, 0 do
        for k = -1, 1 do
            local current_node = {
                x = pos.x + i,
                y = pos.y + j,
                z = pos.z + k
            }
            if(math.abs(i) + math.abs(j) + math.abs(k) == 1) then
                if(minetest.env:get_node(current_node).name == "default:tree") then
                trunk_count = trunk_count + 1
                elseif(minetest.env:get_node(current_node).name ==
                "default:jungletree") then
                    jungle_trunk_count = jungle_trunk_count + 1
                elseif(minetest.env:get_node(current_node).name ==
                "nature:irontree") then
                    iron_trunk_count = iron_trunk_count + 1
                end
            end
        end
        end
        end

        local all_trunks = trunk_count + jungle_trunk_count + iron_trunk_count
        
        -- If there is at least 1 trunk and there are not many of them...
        if (all_trunks > 0) and (all_trunks < DENSITY) then
            minetest.env:remove_node(pos)
            if(math.random(1, all_trunks) <= trunk_count) then
                minetest.env:add_node(pos, {name = "default:tree"})
            elseif(math.random(1, all_trunks) <= jungle_trunk_count) then
                minetest.env:add_node(pos, {name = "default:jungletree"})
            else
                minetest.env:add_node(pos, {name = "nature:irontree"})
            end
            print ('[nature] A trunk has grown at (' .. pos.x .. ',' .. pos.y .. ',' .. pos.z .. ')')
            for i = -1, 1 do
            for j = -1, 1 do
            for k = -1, 1 do
                local current_node = {
                    x = pos.x + i,
                    y = pos.y + j,
                    z = pos.z + k
                }
                if(minetest.env:get_node(current_node).name == "air") then
                    minetest.env:add_node(current_node, {name = "default:leaves"})
                end
            end
            end
            end
        end
    end
})


dofile(minetest.get_modpath("nature") .. "/cactus.lua")
dofile(minetest.get_modpath("nature") .. "/papyrus.lua")
dofile(minetest.get_modpath("nature") .. "/moss.lua")
dofile(minetest.get_modpath("nature") .. "/blossom.lua")

print "[nature] Mod loaded!"
